@@ -1,7 +1,7 @@
 #' @title Partition group plots
 #' @description Plot occurrence partition groups over an environmental predictor raster.
 #' @param e ENMevaluation object
-#' @param envs RasterStack: environmental predictor variable used to build the models in "e"
+#' @param envs SpatRaster: environmental predictor variable used to build the models in "e"
 #' @param pts matrix / data frame: coordinates for occurrence or background data
 #' @param pts.grp numeric vector: partition groups corresponding to data in "pts"
 #' @param ref.data character: plot occurrences ("occs") or background ("bg"), with default "occs"
@@ -9,6 +9,36 @@
 #' @param return.tbl boolean: if TRUE, return the data frames used to make the ggplot instead of the plot itself
 #' @details This function serves as a quick way to visualize occurrence or background partitions over the extent of an environmental predictor raster.
 #' It can be run with an existing ENMevaluation object, or alternatively with occurrence or background coordinates and the corresponding partitions.
+#' 
+#' @examples
+#' \dontrun{
+#' library(terra)
+#' library(ENMeval)
+#' occs <- read.csv(file.path(system.file(package="predicts"), 
+#'                            "/ex/bradypus.csv"))[,2:3]
+#' envs <- rast(list.files(path=paste(system.file(package="predicts"), 
+#'                                    "/ex", sep=""), pattern="tif$", full.names=TRUE))
+#' bg <- as.data.frame(predicts::backgroundSample(envs, n = 10000))
+#' names(bg) <- names(occs)
+#' 
+#' parts <- get.block(occs, bg, orientation = "lat_lon")
+#' 
+#' # now, plot the partition groups for occurrence and background points
+#' evalplot.grps(envs = envs, pts = occs, pts.grp = parts$occs.grp)
+#' evalplot.grps(envs = envs, pts = bg, pts.grp = parts$bg.grp)
+#' 
+#' # you can also plot with an ENMevaluation object
+#' ps <- list(orientation = "lat_lon")
+#' e <- ENMevaluate(occs, envs, bg, 
+#'                  tune.args = list(fc = c("L","LQ"), rm = 1:3), 
+#'                  partitions = "block", partition.settings = ps, 
+#'                  algorithm = "maxnet", categoricals = "biome", 
+#'                  parallel = TRUE)
+#' 
+#' evalplot.grps(e = e, envs = envs, ref.data = "occs")
+#' evalplot.grps(e = e, envs = envs, ref.data = "bg")
+#' }
+#' 
 #' @export
 
 evalplot.grps <- function(e = NULL, envs, pts = NULL, pts.grp = NULL, ref.data = "occs", pts.size = 1.5, return.tbl = FALSE) {
@@ -16,8 +46,8 @@ evalplot.grps <- function(e = NULL, envs, pts = NULL, pts.grp = NULL, ref.data =
     pts.plot <- switch(ref.data, occs = cbind(e@occs, partition = e@occs.grp),
                        bg = cbind(e@bg, partition = e@bg.grp))  
     if(e@partition.method == "testing") {
-      pts.plot <- pts.plot %>% dplyr::mutate(partition = as.numeric(as.character(partition))) %>% 
-        dplyr::bind_rows(e@occs.testing %>% dplyr::mutate(partition = 1)) %>%
+      pts.plot <- pts.plot |> dplyr::mutate(partition = as.numeric(as.character(partition))) |> 
+        dplyr::bind_rows(e@occs.testing |> dplyr::mutate(partition = 1)) |>
         dplyr::mutate(partition = factor(partition))
     } 
     names(pts.plot)[1:2] <- c("longitude", "latitude")
@@ -34,18 +64,18 @@ evalplot.grps <- function(e = NULL, envs, pts = NULL, pts.grp = NULL, ref.data =
   
   grp.n <- length(unique(pts.plot$partition))
   if(grp.n > 9) {
-    theme.custom <- ggplot2::guides(color = FALSE)
+    theme.custom <- ggplot2::guides(color = "none")
     pt.cols <- rainbow(grp.n)
   }else{
     theme.custom <- NULL
     pt.cols <- RColorBrewer::brewer.pal(9, "Set1")
   }
   
-  if(raster::nlayers(envs) > 1) {
+  if(terra::nlyr(envs) > 1) {
     message("Plotting first raster in stack...")
     envs <- envs[[1]]
   }
-  envs.df <- raster::as.data.frame(envs, xy = TRUE)
+  envs.df <- terra::as.data.frame(envs, xy = TRUE, na.rm = FALSE)
   names(envs.df)[3] <- "value"
   g <- ggplot2::ggplot() + ggplot2::geom_raster(data = envs.df, ggplot2::aes(x = x, y = y, fill = value)) +
     ggplot2::geom_point(data = pts.plot, ggplot2::aes(x = longitude, y = latitude, color = partition), size = pts.size) +
@@ -54,16 +84,22 @@ evalplot.grps <- function(e = NULL, envs, pts = NULL, pts.grp = NULL, ref.data =
     ggplot2::coord_equal() + theme.custom
   
   if(return.tbl == TRUE) {
-    return(list(envs.df = envs.df, pts.plot = pts.plot))
+    return(list(envs.df = tibble::as_tibble(envs.df), pts.plot = tibble::as_tibble(pts.plot)))
   }else{
     return(g)  
   }
 }
 
 
-
-
-plot.sim.dataPrep <- function(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data, categoricals, occs.testing.z, quiet) {
+#' Internal plotting function
+#'
+#' This function preps data for plotting.
+#'
+#' @examples \dontrun{
+#' plot.sim.dataPrep()
+#' }
+#' @keywords internal
+plot.sim.dataPrep <- function(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data, occs.testing.z, quiet) {
   
   if(!is.null(e) & any(!is.null(occs.z), !is.null(bg.z), !is.null(occs.grp), !is.null(bg.grp))) {
     stop("* If inputting an ENMevaluation object, leave occs.z, bg.z, occs.grp, and bg.grp NULL. These are read from the object.")
@@ -99,25 +135,25 @@ plot.sim.dataPrep <- function(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data,
   if(ref.data == "bg" & length(unique(bg.grp)) == 1) stop('If background is not partitioned (non-spatial), do not assign ref.data to "bg".')
   
   if(any(is.null(occs.z), is.null(occs.grp))) {
-    pts.plot <- bg.z %>% dplyr::mutate(type = rep(0, nrow(bg.z)), partition = factor(bg.grp))  
+    pts.plot <- bg.z |> dplyr::mutate(type = rep(0, nrow(bg.z)), partition = factor(bg.grp))  
   }else if(any(is.null(bg.z), is.null(bg.grp))) {
-    pts.plot <- occs.z %>% dplyr::mutate(type = rep(1, nrow(occs.z)), partition = factor(occs.grp))  
+    pts.plot <- occs.z |> dplyr::mutate(type = rep(1, nrow(occs.z)), partition = factor(occs.grp))  
   }else{
-    pts.plot <- rbind(occs.z, bg.z) %>% as.data.frame() %>%
+    pts.plot <- rbind(occs.z, bg.z) |> as.data.frame() |>
       dplyr::mutate(type = c(rep(1, nrow(occs.z)), rep(0, nrow(bg.z))), partition = factor(c(occs.grp, bg.grp)))
   }
   names(pts.plot)[1:2] <- c("longitude","latitude")
   
   # find factor rasters or columns and identify them as categoricals
   if(!is.null(envs)) {
-    categoricals <- unique(c(categoricals, names(envs)[which(raster::is.factor(envs))]))
+    categoricals <- unique(names(envs)[which(terra::is.factor(envs))])
     if(length(categoricals) == 0) categoricals <- NULL
   }else{
-    categoricals <- unique(c(categoricals, names(occs.z)[which(sapply(occs.z, is.factor))]))
+    categoricals <- unique(names(occs.z)[which(sapply(occs.z, is.factor))])
     if(length(categoricals) == 0) categoricals <- NULL
   }
   
-  # if categoricals argument was specified, convert these columns to factor class
+  # remove categorical variables for plotting
   if(!is.null(categoricals)) {
     for(i in 1:length(categoricals)) {
       if(!quiet) message(paste0("* Ignoring categorical variable ", categoricals[i], "..."))
@@ -130,11 +166,11 @@ plot.sim.dataPrep <- function(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data,
     if(is.null(e) & is.null(occs.testing.z)) stop("If using fully withheld testing data, input either an ENMevaluation object or occs.testing.z.")
     if(!is.null(e)) occs.testing.z <- e@occs.testing
     names(occs.testing.z)[1:2] <- c("longitude","latitude")
-    occs.testing.z[[categoricals]] <- NULL
-    occs.testing.z <- occs.testing.z %>% dplyr::mutate(type = 1, partition = 2)
+    if(!is.null(categoricals)) occs.testing.z[[categoricals]] <- NULL
+    occs.testing.z <- occs.testing.z |> dplyr::mutate(type = 1, partition = 2)
     pts.plot$partition <- as.numeric(as.character(pts.plot$partition))
     pts.plot[pts.plot$type == 1, "partition"] <- 1
-    pts.plot <- dplyr::bind_rows(pts.plot, occs.testing.z) %>% dplyr::mutate(partition = factor(partition))
+    pts.plot <- dplyr::bind_rows(pts.plot, occs.testing.z) |> dplyr::mutate(partition = factor(partition))
   }
   
   return(pts.plot)
@@ -157,9 +193,6 @@ plot.sim.dataPrep <- function(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data,
 #' the first two columns must be named "longitude" and "latitude"
 #' @param bg.grp numeric vector: partition groups for background records (optional)
 #' @param ref.data character: the reference to calculate MESS based on occurrences ("occs") or background ("bg"), with default "occs"
-#' @param sim.type character: either "mess" for Multivariate Environmental Similarity Surface, "most_diff" for most different variable,
-#' or "most_sim" for most similar variable; uses similarity function from package rmaxent
-#' @param categoricals character vector: names of categorical variables in input RasterStack or data frames to be removed from the analysis;
 #' these must be specified as this function was intended for use with continuous data only; these must be specified when inputting tabular data instead of an ENMevaluation object 
 #' @param envs.vars character vector: names of a predictor variable to plot similarities for; if left NULL, calculations are done
 #' with respect to all variables (optional) 
@@ -169,35 +202,55 @@ plot.sim.dataPrep <- function(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data,
 #' @param return.tbl boolean: if TRUE, return the data frames of similarity values used to make the ggplot instead of the plot itself
 #' @param quiet boolean: if TRUE, silence all function messages (but not errors)
 #' @details Histograms are plotted showing the environmental similarity estimates for each 
-#' partition group. The similarity between environmental values associated with the 
+#' partition group in relation to the others. The similarity between environmental values associated with the 
 #' validation occurrence or background records per partition group and those associated with 
-#' the remaining data (occurrences and background) are calculated, and the minimum similarity 
-#' per grid is returned. For option "mess", higher negative values indicate greater 
-#' environmental difference between the validation occurrences and the study extent, and higher 
-#' positive values indicate greater similarity. This function uses the `similarity()` function 
-#' from the package `rmaxent` (https://github.com/johnbaums/rmaxent/) to calculate the 
-#' similarities. Please see the below reference for details on MESS. 
+#' the remaining data (training occurrences and background) are calculated with the MESS algorithm, and the minimum similarity 
+#' per grid cell is returned. Higher negative values indicate a greater environmental difference between the validation occurrences and the study extent, and higher 
+#' positive values indicate greater similarity. This function uses the `mess()` function 
+#' from the package `predicts`. Please see the below reference for details on MESS.
 #' @return A ggplot of environmental similarities between the occurrence or background data 
 #' for each partition and the rest of the data (all other occurrences and background data).
 #' @references 
 #' Baumgartner J, Wilson P (2021). _rmaxent: Tools for working with Maxent in R_. R package version 0.8.5.9000, <URL: https://github.com/johnbaums/rmaxent>.
 #' Elith, J., Kearney, M., and Phillips, S. (2010) The art of modelling range-shifting species. \emph{Methods in Ecology and Evolution}, \bold{1}: 330-342. \doi{doi:10.1111/j.2041-210X.2010.00036.x}
 #' 
+#' @examples
+#' \dontrun{
+#' # first, let's tune some models
+#' occs <- read.csv(file.path(system.file(package="predicts"), 
+#' "/ex/bradypus.csv"))[,2:3]
+#' envs <- rast(list.files(path=paste(system.file(package="predicts"), 
+#' "/ex", sep=""), pattern="tif$", full.names=TRUE))
+#' bg <- as.data.frame(predicts::backgroundSample(envs, n = 10000))
+#' names(bg) <- names(occs)
+#'  
+#' ps <- list(orientation = "lat_lat")
+#' 
+#' e <- ENMevaluate(occs, envs, bg, 
+#'                tune.args = list(fc = c("L","LQ","LQH"), rm = 1:5), 
+#'                partitions = "block", partition.settings = ps, 
+#'                algorithm = "maxnet", categoricals = "biome", 
+#'                parallel = TRUE)
+#' 
+#' # now, plot the environmental similarity of each partition to the others               
+#' evalplot.envSim.hist(e)
+#' }
+#' 
 #' @export
 
 evalplot.envSim.hist <- function(e = NULL, occs.z = NULL, bg.z = NULL, occs.grp = NULL, 
-                                 bg.grp = NULL, ref.data = "occs", sim.type = "mess", 
-                                 categoricals = NULL, envs.vars = NULL, occs.testing.z = NULL,
+                                 bg.grp = NULL, ref.data = "occs", 
+                                 envs.vars = NULL, occs.testing.z = NULL,
                                  hist.bins = 30, return.tbl = FALSE, quiet = FALSE) {
   
-  pts.plot <- plot.sim.dataPrep(e, envs = NULL, occs.z, bg.z, occs.grp, bg.grp, ref.data, categoricals, occs.testing.z, quiet)
+  pts.plot <- plot.sim.dataPrep(e, envs = NULL, occs.z, bg.z, occs.grp, bg.grp, ref.data, occs.testing.z, quiet)
   
-  envs.names <- pts.plot %>% dplyr::select(-longitude, -latitude, -partition, -type) %>% names()
+  envs.names <- pts.plot |> dplyr::select(-longitude, -latitude, -partition, -type) |> names()
   
   if(!is.null(envs.vars)) {
     if(!quiet) message(paste0("* Similarity values calculated based only on ", paste(envs.vars, collapse = ", "), "."))
     envs.rem <- envs.names[-which(envs.names %in% envs.vars)]
-    pts.plot <- pts.plot %>% dplyr::select(-dplyr::all_of(envs.rem))
+    pts.plot <- pts.plot |> dplyr::select(-dplyr::all_of(envs.rem))
   }
   
   test.sim <- list()
@@ -207,29 +260,27 @@ evalplot.envSim.hist <- function(e = NULL, occs.z = NULL, bg.z = NULL, occs.grp
   }
   
   for(k in 1:nk) {
-    test.z <- pts.plot %>% dplyr::filter(partition == k) %>% dplyr::select(-longitude, -latitude, -partition)
+    test.z <- pts.plot |> dplyr::filter(partition == k) |> dplyr::select(-longitude, -latitude, -partition)
     if(ref.data == "occs") {
-      test.z <- test.z %>% dplyr::filter(type == 1) %>% dplyr::select(-type)
+      test.z <- test.z |> dplyr::filter(type == 1) |> dplyr::select(-type)
     }else if (ref.data == "bg") {
-      test.z <- test.z %>% dplyr::filter(type == 0) %>% dplyr::select(-type)
+      test.z <- test.z |> dplyr::filter(type == 0) |> dplyr::select(-type)
     }
-    train.z <- pts.plot %>% dplyr::filter(partition != k) %>% dplyr::select(-longitude, -latitude, -partition, -type)
+    train.z <- pts.plot |> dplyr::filter(partition != k) |> dplyr::select(-longitude, -latitude, -partition, -type)
     
     sim <- tryCatch({
-      similarity(train.z, test.z)
+      predicts::mess(train.z, test.z)
     }, error = function(cond) {
-      message('Error: there may be a categorical variable present in the predictor variable data. Please make sure to declare all categorical variables with the "categoricals" argument.')
+      message('Error: there may be at least one categorical variable in the predictor data that is not attributed as a factor. Please convert these variable(s) to factor.')
       # Choose a return value in case of error
       return(NULL)
     })
-    sim.sel <- switch(sim.type, mess = sim$similarity_min, most_diff = sim$mod, most_sim = sim$mos)  
-    
-    test.sim[[k]] <- data.frame(partition = k, sim.sel)
-    names(test.sim[[k]])[2] <- sim.type  
+    test.sim[[k]] <- data.frame(partition = k, sim)
+    names(test.sim[[k]])[2] <- "mess"  
   }
   
   if(nk > 9) {
-    theme.custom <- ggplot2::guides(color = FALSE)
+    theme.custom <- ggplot2::guides(color = "none")
     pt.cols <- rainbow(nk)
   }else{
     theme.custom <- NULL
@@ -243,37 +294,17 @@ evalplot.envSim.hist <- function(e = NULL, occs.z = NULL, bg.z = NULL, occs.grp
                      switch(ref.data, occs = "occurrence", bg = "background"),
                      "partitions and all other background partitions.)")
   
-  if(sim.type != "mess") {
-    envs.tbl <- data.frame(sort(unique(plot.df[,2])), envs.names)
-    names(envs.tbl) <- c(sim.type, "env.var")
-    envs.tbl$env.var <- factor(envs.tbl$env.var)
-    plot.df <- plot.df %>% dplyr::left_join(envs.tbl, by = sim.type)
-    plot.df[[sim.type]] <- NULL
-    names(plot.df)[2] <- sim.type
-    title <- paste(switch(sim.type, most_diff = "Most different", most_sim = "Most similar"), "environmental variable")
-    
-    g <- ggplot2::ggplot(plot.df, ggplot2::aes_string(x = sim.type, fill = "partition")) + 
-      ggplot2::stat_count() +
-      ggplot2::facet_grid(ggplot2::vars(partition)) + 
-      ggplot2::scale_fill_manual(values = pt.cols) +
-      ggplot2::theme_classic() +
-      ggplot2::geom_vline(xintercept = 0) +
-      ggplot2::ggtitle(paste(title, plot.text, collapse = "\n")) +
-      ggplot2::theme(strip.background = ggplot2::element_blank(), strip.text.y = ggplot2::element_blank())
-  }else if(sim.type == "mess"){
-    g <- ggplot2::ggplot(plot.df, ggplot2::aes(x = mess, fill = partition)) + 
-      ggplot2::geom_histogram(bins = hist.bins) +
-      ggplot2::facet_grid(ggplot2::vars(partition)) + 
-      ggplot2::scale_fill_manual(values = pt.cols) +
-      ggplot2::theme_classic() +
-      ggplot2::geom_vline(xintercept = 0) +
-      ggplot2::ggtitle(paste("Multivariate environmental similarity", plot.text, collapse = "\n")) +
-      ggplot2::theme(strip.background = ggplot2::element_blank(), strip.text.y = ggplot2::element_blank())  
-  }
-  g
+  g <- ggplot2::ggplot(plot.df, ggplot2::aes(x = mess, fill = partition)) + 
+    ggplot2::geom_histogram(bins = hist.bins) +
+    ggplot2::facet_grid(ggplot2::vars(partition)) + 
+    ggplot2::scale_fill_manual(values = pt.cols) +
+    ggplot2::theme_classic() +
+    ggplot2::geom_vline(xintercept = 0) +
+    ggplot2::ggtitle(paste("Multivariate environmental similarity", plot.text, collapse = "\n")) +
+    ggplot2::theme(strip.background = ggplot2::element_blank(), strip.text.y = ggplot2::element_blank())  
   
   if(return.tbl == TRUE) {
-    return(plot.df)
+    return(tibble::as_tibble(plot.df))
   }else{
     return(g)  
   }
@@ -289,8 +320,8 @@ evalplot.envSim.hist <- function(e = NULL, occs.z = NULL, bg.z = NULL, occs.grp
 #' object or the argument occs.testing.z. In the resulting plot, partition 1 refers to the training data,
 #' while partition 2 refers to the fully withheld testing group.
 #' @param e ENMevaluation object (optional) 
-#' @param envs RasterStack: environmental predictor variables used to build the models in "e"; categorical variables should be 
-#' removed before input or identified with the argument "categoricals", as they cannot be used to calculate MESS
+#' @param envs SpatRaster: environmental predictor variables used to build the models in "e"; categorical variables will be 
+#' removed before internally as they cannot be used to calculate MESS
 #' @param occs.z data frame: longitude, latitude, and environmental predictor variable values for occurrence records, in that order (optional);
 #' the first two columns must be named "longitude" and "latitude"
 #' @param occs.grp numeric vector: partition groups for occurrence records (optional)
@@ -298,10 +329,6 @@ evalplot.envSim.hist <- function(e = NULL, occs.z = NULL, bg.z = NULL, occs.grp
 #' the first two columns must be named "longitude" and "latitude"
 #' @param bg.grp numeric vector: partition groups for background records (optional)
 #' @param ref.data character: the reference to calculate MESS based on occurrences ("occs") or background ("bg"), with default "occs"
-#' @param sim.type character: either "mess" for Multivariate Environmental Similarity Surface, "most_diff" for most different variable,
-#' or "most_sim" for most similar variable; uses similarity function from package rmaxent
-#' @param categoricals character vector: names of categorical variables in input RasterStack or data frames to be removed from the analysis;
-#' these must be specified as this function was intended for use with continuous data only
 #' @param envs.vars character vector: names of a predictor variable to plot similarities for; if left NULL, calculations are done
 #' with respect to all variables (optional) 
 #' @param bb.buf numeric: distance used to buffer (extend) the mapping extent in map units; for latitude/longitude, this is in degrees (optional)
@@ -313,17 +340,16 @@ evalplot.envSim.hist <- function(e = NULL, occs.z = NULL, bg.z = NULL, occs.grp
 #' @param gradient.colors character vector: colors used for ggplot2::scale_fill_gradient2
 #' @param na.color character: color used for NA values
 #' @param return.tbl boolean: if TRUE, return the data frames of similarity values used to make the ggplot instead of the plot itself
-#' @param return.ras boolean: if TRUE, return the RasterStack of similarity values used to make the ggplot instead of the plot itself
+#' @param return.ras boolean: if TRUE, return the SpatRaster of similarity values used to make the ggplot instead of the plot itself
 #' @param quiet boolean: if TRUE, silence all function messages (but not errors)
 #' @details Rasters are plotted showing the environmental similarity estimates for each 
-#' partition group. The similarity between environmental values associated with the 
+#' partition group in relation to the others. The similarity between environmental values associated with the 
 #' validation occurrence or background records per partition group and those associated with 
-#' the entire study extent (specified by the extent of the input RasterStack "envs") are 
-#' calculated, and the minimum similarity per grid is returned. For option "mess", higher 
+#' the entire study extent (specified by the extent of the input SpatRaster "envs") are 
+#' calculated with the MESS algorithm, and the minimum similarity per grid cell is returned. Higher 
 #' negative values indicate greater environmental difference between the validation occurrences 
 #' and the study extent, and higher positive values indicate greater similarity. This function 
-#' uses the `similarity()` function from the package `rmaxent` 
-#' (https://github.com/johnbaums/rmaxent/) to calculate the similarities. Please see the below 
+#' uses the `mess()` function from the package `predicts` to calculate the similarities. Please see the below 
 #' reference for details on MESS. 
 #' @return A ggplot of environmental similarities between the occurrence or background data 
 #' for each partition and all predictor variable values in the extent.
@@ -331,11 +357,36 @@ evalplot.envSim.hist <- function(e = NULL, occs.z = NULL, bg.z = NULL, occs.grp
 #' Baumgartner J, Wilson P (2021). _rmaxent: Tools for working with Maxent in R_. R package version 0.8.5.9000, <URL: https://github.com/johnbaums/rmaxent>.
 #' Elith, J., Kearney, M., and Phillips, S. (2010) The art of modelling range-shifting species. \emph{Methods in Ecology and Evolution}, \bold{1}: 330-342. \doi{doi:10.1111/j.2041-210X.2010.00036.x}
 #' 
+#' @examples
+#' \dontrun{
+#' library(terra)
+#' library(ENMeval)
+#' 
+#' # first, let's tune some models
+#' occs <- read.csv(file.path(system.file(package="predicts"), 
+#' "/ex/bradypus.csv"))[,2:3]
+#' envs <- rast(list.files(path=paste(system.file(package="predicts"), 
+#' "/ex", sep=""), pattern="tif$", full.names=TRUE))
+#' bg <- as.data.frame(predicts::backgroundSample(envs, n = 10000))
+#' names(bg) <- names(occs)
+#'  
+#' ps <- list(orientation = "lat_lat")
+#' 
+#' e <- ENMevaluate(occs, envs, bg, 
+#'                tune.args = list(fc = c("L","LQ","LQH"), rm = 1:5), 
+#'                partitions = "block", partition.settings = ps, 
+#'                algorithm = "maxnet", categoricals = "biome", 
+#'                parallel = TRUE)
+#' 
+#' # now, plot the environmental similarity of each partition to the others               
+#' evalplot.envSim.map(e, envs)
+#' }
+#' 
 #' @export
 
 evalplot.envSim.map <- function(e = NULL, envs, occs.z = NULL, bg.z = NULL, occs.grp = NULL, 
-                                bg.grp = NULL, ref.data = "occs", sim.type = "mess", 
-                                categoricals = NULL, envs.vars = NULL, bb.buf = 0, occs.testing.z = NULL,
+                                bg.grp = NULL, ref.data = "occs", 
+                                envs.vars = NULL, bb.buf = 0, occs.testing.z = NULL,
                                 plot.bg.pts = FALSE, sim.palette = NULL, 
                                 pts.size = 1.5, gradient.colors = c("red","white","blue"), na.color = "gray",
                                 return.tbl = FALSE, return.ras = FALSE, quiet = FALSE) {
@@ -353,22 +404,26 @@ evalplot.envSim.map <- function(e = NULL, envs, occs.z = NULL, bg.z = NULL, occs
   
   if(!is.numeric(bb.buf)) stop("Please ensure bb.buf is a number.")
   
-  pts.plot <- plot.sim.dataPrep(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data, categoricals, occs.testing.z, quiet)
+  pts.plot <- plot.sim.dataPrep(e, envs, occs.z, bg.z, occs.grp, bg.grp, ref.data, occs.testing.z, quiet)
   
-  if(!is.null(categoricals) & !is.null(envs)) envs <- raster::dropLayer(envs, categoricals)
+  categoricals <- unique(names(envs)[which(terra::is.factor(envs))])
+  if(length(categoricals) != 0 & !is.null(envs)) {
+    envs <- terra::subset(envs, -which(names(envs) %in% categoricals))
+  }
   
   if(!is.null(envs.vars)) {
-    if(!quiet) message(paste0("* Similarity values calculated based only on ", paste(envs.vars, collapse = ", "), "."))
+    if(!quiet) message(paste0("* Similarity values calculated based only on ", 
+                              paste(envs.vars, collapse = ", "), "."))
     envs.names <- names(envs)
     envs.rem <- envs.names[-which(envs.names %in% envs.vars)]
-    pts.plot <- pts.plot %>% dplyr::select(-dplyr::all_of(envs.rem))
+    pts.plot <- pts.plot |> dplyr::select(-dplyr::all_of(envs.rem))
     envs <- envs[[envs.vars]]
   }
   
   if(ref.data == "occs") {
-    pts.plot <- pts.plot %>% dplyr::filter(type == 1) %>% dplyr::select(-type)
+    pts.plot <- pts.plot |> dplyr::filter(type == 1) |> dplyr::select(-type)
   }else if (ref.data == "bg") {
-    pts.plot <- pts.plot %>% dplyr::filter(type == 0) %>% dplyr::select(-type)
+    pts.plot <- pts.plot |> dplyr::filter(type == 0) |> dplyr::select(-type)
   }
   
   ras.sim <- list()
@@ -378,43 +433,39 @@ evalplot.envSim.map <- function(e = NULL, envs, occs.z = NULL, bg.z = NULL, occs
   }
   
   for(k in 1:nk) {
-    test.z <- pts.plot %>% dplyr::filter(partition == k) %>% dplyr::select(-longitude, -latitude, -partition)
+    message("Calculating MESS for partition ", k, "...")
+    test.z <- pts.plot |> dplyr::filter(partition == k) |> 
+      dplyr::select(-longitude, -latitude, -partition)
     
     sim <- tryCatch({
-      similarity(envs, test.z)
+      predicts::mess(envs, test.z)
     }, error = function(cond) {
-      message('Error: there may be a categorical variable present in the predictor variable data. Please make sure to declare all categorical variables with the "categoricals" argument.')
+      message('Error: there may be at least one categorical variable in the predictor data that is not attributed as a factor. Please convert these variable(s) to factor.')
       # Choose a return value in case of error
       return(NULL)
     })
-    sim.sel <- switch(sim.type, mess = sim$similarity_min, most_diff = sim$mod, most_sim = sim$mos)  
+    names(sim) <- paste0("partition", k)
     
-    ras.sim[[k]] <- sim.sel
+    ras.sim[[k]] <- sim
   }
   
-  rs.sim <- raster::stack(ras.sim)
-  names(rs.sim) <- gsub("layer|mess", "partition", names(rs.sim))
-  plot.df <- raster::as.data.frame(rs.sim, xy = TRUE) %>%
-    tidyr::pivot_longer(cols = 3:ncol(.), names_to = "ras", values_to = sim.type)
+  rs.sim <- terra::rast(ras.sim)
+  plot.df <- terra::as.data.frame(rs.sim, xy = TRUE, na.rm = FALSE) |>
+    tidyr::pivot_longer(cols = 3:dplyr::last_col(), names_to = "ras", values_to = "mess")
   # add buffer
-  plot.df <- plot.df %>% dplyr::filter(x > min(pts.plot$longitude) - bb.buf, x < max(pts.plot$longitude) + bb.buf,
-                                       y > min(pts.plot$latitude) - bb.buf, y < max(pts.plot$latitude) + bb.buf)
+  plot.df <- plot.df |> dplyr::filter(x > min(pts.plot$longitude) - bb.buf, 
+                                      x < max(pts.plot$longitude) + bb.buf,
+                                      y > min(pts.plot$latitude) - bb.buf, 
+                                      y < max(pts.plot$latitude) + bb.buf)
   
-  if(sim.type != "mess") {
-    if(is.null(sim.palette)) sim.palette <- "Set1"
-    plot.df$ras <- gsub("_var", "", plot.df$ras)
-    plot.df[[sim.type]] <- factor(plot.df[[sim.type]])
-    title <- paste(switch(sim.type, most_diff = "Most different", most_sim = "Most similar"), "environmental variable")
-  }else{
-    title <- "Multivariate environmental similarity" 
-  }
+  title <- "Multivariate environmental similarity" 
   
   plot.text <- paste("\n(Values represent environmental similarity between", 
                      switch(ref.data, occs = "occurrence", bg = "background"),
                      "partitions and all raster cells with values.)")
   
   g <- ggplot2::ggplot() +
-    ggplot2::geom_raster(data = plot.df, ggplot2::aes_string(x = "x", y = "y", fill = sim.type))
+    ggplot2::geom_raster(data = plot.df, ggplot2::aes_string(x = "x", y = "y", fill = "mess"))
   if(ref.data == "bg" & plot.bg.pts == FALSE) {
     g <- g + ggplot2::facet_wrap(ggplot2::vars(ras), ncol = 2) +
       ggplot2::ggtitle(paste(title, plot.text, collapse = "\n")) +
@@ -425,15 +476,10 @@ evalplot.envSim.map <- function(e = NULL, envs, occs.z = NULL, bg.z = NULL, occs
       ggplot2::ggtitle(paste(title, plot.text, collapse = "\n")) +
       ggplot2::theme_classic()
   }
-  if(sim.type != "mess") {
-    g <- g + ggplot2::scale_fill_brewer(palette = sim.palette, na.value = "white", breaks = levels(plot.df[[sim.type]]))
-  }else{
-    g <- g + ggplot2::scale_fill_gradient2(low = gradient.colors[1], mid = gradient.colors[2], high = gradient.colors[3], na.value = na.color)
-  }
-  g
+  g <- g + ggplot2::scale_fill_gradient2(low = gradient.colors[1], mid = gradient.colors[2], high = gradient.colors[3], na.value = na.color)
   
   if(return.tbl == TRUE) {
-    return(plot.df)
+    return(tibble::as_tibble(plot.df))
   }else if(return.ras == TRUE) {
     return(rs.sim)
   }else{
@@ -459,26 +505,50 @@ evalplot.envSim.map <- function(e = NULL, envs, occs.z = NULL, bg.z = NULL, occs
 #' Error bars represent the standard deviation of a statistic around the mean. 
 #' Currently, this function can only plot two tuning parameters at a time.
 #' @return A ggplot of evaluation statistics. 
+#' 
+#' @examples
+#' \dontrun{
+#' # first, let's tune some models
+#' occs <- read.csv(file.path(system.file(package="predicts"), 
+#' "/ex/bradypus.csv"))[,2:3]
+#' envs <- rast(list.files(path=paste(system.file(package="predicts"), 
+#' "/ex", sep=""), pattern="tif$", full.names=TRUE))
+#' bg <- as.data.frame(predicts::backgroundSample(envs, n = 10000))
+#' names(bg) <- names(occs)
+#'  
+#' ps <- list(orientation = "lat_lat")
+
+#' e <- ENMevaluate(occs, envs, bg, 
+#'                tune.args = list(fc = c("L","LQ","LQH"), rm = 1:5), 
+#'                partitions = "block", partition.settings = ps, 
+#'                algorithm = "maxnet", categoricals = "biome", 
+#'                parallel = TRUE)
+#'                
+#' evalplot.stats(e, c("cbi.val", "or.mtp"), x.var = "rm", color.var = "fc", 
+#'              error.bars = FALSE)
+#' }
 #' @export
 
 evalplot.stats <- function(e, stats, x.var, color.var, dodge = NULL, error.bars = TRUE, facet.labels = NULL, metric.levels = NULL, return.tbl = FALSE) {
   exp <- paste(paste0("*", stats), collapse = "|")
-  res <- e@results %>% 
-    tidyr::pivot_longer(cols = auc.train:ncoef, names_to = "metric", values_to = "value") %>%
+  res <- e@results |> 
+    tidyr::pivot_longer(cols = auc.train:ncoef, names_to = "metric", values_to = "value") |>
     dplyr::filter(grepl(exp, metric))
-  avgs <- res %>% 
-    dplyr::filter(grepl("avg", metric)) %>%
-    dplyr::rename(avg = value) %>%
+  avgs <- res |> 
+    dplyr::filter(grepl("avg", metric)) |>
+    dplyr::rename(avg = value) |>
     dplyr::mutate(metric = gsub(".avg", "", metric))
-  sds <- res %>% 
-    dplyr::filter(grepl("sd", metric)) %>%
-    dplyr::rename(sd = value) %>%
+  sds <- res |> 
+    dplyr::filter(grepl("sd", metric)) |>
+    dplyr::rename(sd = value) |>
     dplyr::mutate(metric = gsub(".sd", "", metric))
   join.names <- names(avgs)
   join.names <- join.names[join.names != "avg"]
-  res.avgs <- dplyr::left_join(avgs, sds, by = join.names) %>%
+  res.avgs <- dplyr::left_join(avgs, sds, by = join.names) |>
     dplyr::mutate(lower = avg - sd, upper = avg + sd,
                   metric = factor(metric, levels = stats))
+  res.avgs[[x.var]] <- factor(res.avgs[[x.var]])
+  res.avgs[[color.var]] <- factor(res.avgs[[color.var]])
   if(!is.null(facet.labels)) labeller <- ggplot2::as_labeller(facet.labels) else labeller <- NULL
   if(!is.null(metric.levels)) res$metric <- factor(res$metric, levels = metric.levels)
   if(!is.null(metric.levels)) res.avgs$metric <- factor(res.avgs$metric, levels = metric.levels)
@@ -502,7 +572,7 @@ evalplot.stats <- function(e, stats, x.var, color.var, dodge = NULL, error.bars
     }
     g
     if(return.tbl == TRUE) {
-      return(res.avgs)
+      return(tibble::as_tibble(res.avgs))
     }else{
       return(g)  
     }
@@ -519,7 +589,7 @@ evalplot.stats <- function(e, stats, x.var, color.var, dodge = NULL, error.bars
     }
     g
     if(return.tbl == TRUE) {
-      return(res)
+      return(tibble::as_tibble(res))
     }else{
       return(g)  
     }
@@ -540,32 +610,67 @@ evalplot.stats <- function(e, stats, x.var, color.var, dodge = NULL, error.bars
 #' the quantiles and the empirical value is plotted as a red point along the vertical axis. 
 #' For histogram plots, the null distribution is displayed as a histogram with vertical lines showing the quantiles 
 #' and the empirical value as a vertical red line on the distribution.
+#' 
+#' @examples
+#' \dontrun{
+#' # first, let's tune some models
+#' occs <- read.csv(file.path(system.file(package="predicts"), 
+#'                            "/ex/bradypus.csv"))[,2:3]
+#' envs <- rast(list.files(path=paste(system.file(package="predicts"), 
+#'                                    "/ex", sep=""), pattern="tif$", full.names=TRUE))
+#' bg <- as.data.frame(predicts::backgroundSample(envs, n = 10000))
+#' names(bg) <- names(occs)
+#' 
+#' ps <- list(orientation = "lat_lat")
+#' 
+#' e <- ENMevaluate(occs, envs, bg, 
+#'                  tune.args = list(fc = c("L","LQ","LQH"), rm = 2:4), partitions = "block", 
+#'                  partition.settings = ps, algorithm = "maxnet", categoricals = "biome",
+#'                  parallel = TRUE)
+#' 
+#' d <- eval.results(e)
+#' 
+#' # here, we will choose an optimal model based on validation CBI, but you can
+#' # choose yourself what evaluation statistics to use
+#' opt <- d |> filter(cbi.val.avg == max(cbi.val.avg))
+#' 
+#' # now we can run our null models 
+#' # NOTE: you should use at least 100 iterations in practice -- this is just an
+#' # example
+#' nulls <- ENMnulls(e, mod.settings = list(fc = opt$fc, rm = opt$rm), no.iter = 10)
+#' 
+#' # let's plot the null model results
+#' evalplot.nulls(nulls, stats = c("cbi.val", "auc.val"), plot.type = "violin")
+#' }
+#' 
 #' @return A ggplot of null model statistics. 
 #' @export
 
 evalplot.nulls <- function(e.null, stats, plot.type, facet.labels = NULL, metric.levels = NULL, return.tbl = FALSE) {
   exp <- paste(paste0("*", stats), collapse = "|")
-  null.res <- e.null@null.results %>% 
-    tidyr::pivot_longer(cols = auc.train:ncoef, names_to = "metric", values_to = "value") %>%
-    dplyr::filter(grepl(exp, metric)) %>%
+  null.res <- e.null@null.results |> 
+    tidyr::pivot_longer(cols = auc.train:ncoef, names_to = "metric", values_to = "value") |>
+    dplyr::filter(grepl(exp, metric)) |>
     dplyr::select(metric, value)
-  null.avgs <- null.res %>% 
-    dplyr::filter(grepl("avg", metric) | metric %in% stats) %>%
-    dplyr::rename(avg = value) %>%
-    dplyr::mutate(metric = gsub(".avg", "", metric))
+  null.avgs <- null.res |> 
+    dplyr::filter(grepl("avg", metric) | metric %in% stats) |>
+    dplyr::rename(avg = value) |>
+    dplyr::mutate(metric = gsub(".avg", "", metric)) |>
+    dplyr::mutate(metric = factor(metric, levels = stats))
   if(!is.null(metric.levels)) null.avgs$metric <- factor(null.avgs$metric, levels = metric.levels)
-  # null.sds <- null.res %>% 
-  #   dplyr::filter(grepl("sd", metric)) %>%
-  #   dplyr::rename(sd = value) %>%
+  # null.sds <- null.res |> 
+  #   dplyr::filter(grepl("sd", metric)) |>
+  #   dplyr::rename(sd = value) |>
   #   dplyr::mutate(metric = gsub(".sd", "", metric))
-  # null.res.avgs <- dplyr::bind_cols(null.avgs, null.sds %>% dplyr::select(sd))
+  # null.res.avgs <- dplyr::bind_cols(null.avgs, null.sds |> dplyr::select(sd))
   
-  emp.res <- e.null@null.emp.results %>% 
-    dplyr::slice(1) %>%
-    tidyr::pivot_longer(cols = stats, names_to = "metric", values_to = "value") %>%
-    dplyr::select(statistic, metric, value) %>%
-    tidyr::pivot_wider(names_from = statistic, values_from = value) %>%
-    dplyr::rename(avg = emp.mean)
+  emp.res <- e.null@null.emp.results |> 
+    dplyr::slice(1) |>
+    tidyr::pivot_longer(cols = stats, names_to = "metric", values_to = "value") |>
+    dplyr::select(statistic, metric, value) |>
+    tidyr::pivot_wider(names_from = statistic, values_from = value) |>
+    dplyr::rename(avg = emp.mean) |>
+    dplyr::mutate(metric = factor(metric, levels = stats))
   
   if(!is.null(facet.labels)) labeller <- ggplot2::as_labeller(facet.labels) else labeller <- NULL
   
@@ -576,14 +681,14 @@ evalplot.nulls <- function(e.null, stats, plot.type, facet.labels = NULL, metric
       ggplot2::theme_bw()  
   }else if(plot.type == "histogram") {
     stats.all <- rbind(null.avgs, emp.res)
-    vlines <- null.avgs %>% dplyr::group_by(metric) %>% 
+    vlines <- null.avgs |> dplyr::group_by(metric) |> 
       dplyr::summarize(`0.01 quantile` = quantile(avg, 0.01),
                        `0.05 quantile` = quantile(avg, 0.05),
                        `0.50 quantile` = quantile(avg, 0.5),
                        `0.95 quantile` = quantile(avg, 0.95),
-                       `0.99 quantile` = quantile(avg, 0.99)) %>%
+                       `0.99 quantile` = quantile(avg, 0.99)) |>
       tidyr::pivot_longer(cols = `0.01 quantile`:`0.99 quantile`, names_to = "quantile", values_to = "value")
-    vlines <- rbind(vlines, emp.res %>% dplyr::mutate(quantile = "empirical value") %>% dplyr::rename(value = avg))
+    vlines <- rbind(vlines, emp.res |> dplyr::mutate(quantile = "empirical value") |> dplyr::rename(value = avg))
     g <- ggplot2::ggplot(mapping = ggplot2::aes(x = avg)) + 
       ggplot2::geom_histogram(data = null.avgs, fill = "gray80") +
       ggplot2::geom_vline(data = vlines, ggplot2::aes(xintercept = value, color = quantile, linetype = quantile, size = quantile)) +
@@ -615,7 +720,7 @@ evalplot.nulls <- function(e.null, stats, plot.type, facet.labels = NULL, metric
     }
   }
   if(return.tbl == TRUE) {
-    return(list(null.avgs = null.avgs, empirical.results = emp.res))
+    return(list(null.avgs = tibble::as_tibble(null.avgs), empirical.results = tibble::as_tibble(emp.res)))
   }else{
     return(g)  
   }
